module MoshGenerator

  #
  # Override Rails default AppBuilder
  #
  class AppBuilder < Rails::AppBuilder

    #
    # Create the README file.
    #
    def readme
      template 'README.md.tt'
    end

    #
    # Create the Gemfile.
    #
    def gemfile
      template 'Gemfile.tt'
    end

    #
    # Create the .gitignore.
    #
    def gitignore
      copy_file '.gitignore'
    end

    #
    # Create the database.yml.
    #
    def database_yml
      copy_file 'config/database.yml'
    end

    #
    # Creates public directory.
    #
    def public_directory
      super
      directory 'public_directory', 'public', recursive: false, force: true
      if options[:skip_sprockets]
        directory 'public_directory/images', 'public/images'
        directory 'public_directory/javascripts', 'public/javascripts'
        directory 'public_directory/stylesheets', 'public/stylesheets'
      end
    end

    #
    # Removes the index.html file on the public directory.
    #
    def remove_public_index
      remove_file 'public/index.html'
    end

    #
    # Removes the assets directories on app, lib and vendor directories.
    #
    def remove_assets_directories
      if options[:skip_sprockets]
        remove_dir 'app/assets'
        remove_dir 'lib/assets'
        remove_dir 'vendor/assets'
      end
    end

    #
    # Removes the doc directory.
    #
    def remove_doc_directory
      remove_dir 'doc'
    end

    #
    # Creates an .editorconfig.
    #
    def create_editorconfig_file
      copy_file '.editorconfig'
    end

    #
    # Creates a .ruby-version file to Ruby version managers.
    #
    def create_ruby_version_file
      template '.ruby-version'
    end

    #
    # Creates a .ruby-gemset file to Ruby version managers.
    #
    def create_ruby_gemset_file
      template '.ruby-gemset'
    end

    #
    # Creates a staging.rb file on config/environments directory.
    #
    def create_staging_environment_file
      run 'cp config/environments/production.rb config/environments/staging.rb'
    end

    #
    # Creates the initial helpers for the application.
    #
    def create_helpers
      template 'app_directory/helpers/application_helper.rb.tt', 'app/helpers/application_helper.rb', force: true
    end

    #
    # Creates the layout for the application.
    #
    def create_layout
      template 'app_directory/views/layouts/application.html.erb.tt', 'app/views/layouts/application.html.erb', force: true
    end

    #
    # Creates the partial views directory with some partial for the application.
    #
    def create_partial_views
      directory 'app_directory/views/application', 'app/views/application'
    end

    #
    # Configures Grunt to compile, concat and minify assets.
    #
    def configure_grunt
      if options[:skip_sprockets]
        template 'package.json.tt'
        copy_file 'Gruntfile.js'
        copy_file '.jshintrc'
        run 'npm install'
      end
    end

  end

end
