require 'rails/generators'
require 'rails/generators/rails/app/app_generator'

module MoshGenerator

  #
  # Override Rails default AppGenerator.
  #
  class AppGenerator < Rails::Generators::AppGenerator

    class << self
      #
      # Banner for the cli.
      #
      def banner
        "mosh_generator #{self.arguments.map(&:usage).join(' ')} [options]"
      end
    end

    # Override skip_test_unit option defaults to true.
    class_option :skip_test_unit, :type => :boolean, :aliases => '-T',
      :default => true, :desc => 'Skip Test::Unit files'

    # Override skip_sprockets option defaults to true.
    class_option :skip_sprockets, :type => :boolean, :aliases => '-S',
      :default => true, :desc => 'Skip Sprockets files'

    # Override skip_javascript option defaults to true.
    class_option :skip_javascript, :type => :boolean, :aliases => '-J',
      :default => true, :desc => 'Skip JavaScript files'

    #
    # Overrides default implementation to run customization code.
    #
    def finish_template
      invoke :mosh_generator_customization
      super
    end

    #
    # Customized generator wrapper.
    #
    def mosh_generator_customization
      invoke :remove_files_we_dont_need
      invoke :create_configuration_files
      invoke :create_staging_environment
      invoke :create_application_structure
    end

    #
    # Do some clean up.
    #
    def remove_files_we_dont_need
      build :remove_public_index
      build :remove_assets_directories
      build :remove_doc_directory
    end

    #
    # Creates some configuration files for tools.
    #
    def create_configuration_files
      build :create_editorconfig_file
      build :create_ruby_version_file
      build :create_ruby_gemset_file
    end

    #
    # Creates a staging environment
    #
    def create_staging_environment
      build :create_staging_environment_file
    end

    #
    # Create the minimal application structure.
    #
    def create_application_structure
      build :create_helpers
      build :create_layout
      build :create_partial_views
      build :configure_grunt
    end

    #
    # Override the original run_bundle to do nothing.
    #
    def run_bundle
      bundle_command 'install --without production staging'
    end

    protected

      #
      # Override get_builder_class to get builder class from MoshGenerator module.
      #
      # @return [MoshGenerator::AppBuilder] Mosh Generator's AppBuilder implementation.
      #
      def get_builder_class
        MoshGenerator::AppBuilder
      end
  end

end
